/**
 * @(#)Album.java  10 November 2007
 * Lab 9
 * @author Viera K. Proulx
 */
import java.util.ArrayList;
import interfaces.*;

/**
 * Class to represent a traversal over a list of recording albums
 * -- using ArrayList as the list data structure
 */
public class AlbumListTraversal implements Traversal<Album> {
  private ArrayList<Album> alist;
  private int current;
  
  /**
   * The full constructor
   * @param alist the <CODE>{@link ArrayList ArrayList}</CODE>
   * to traverse
   * @param current current for the first element in this traversal
   */
  protected AlbumListTraversal(ArrayList<Album> alist, int current){
    this.alist = alist;
    if (current <= this.alist.size()) 
      this.current = current;
    else 
      throw new RuntimeException("invalid index in the constructor");
  }
  
  /**
   * The initializing constructor
   * -- used to start the iterator for a given ArrayList
   * @param alist the <CODE>{@link ArrayList ArrayList}</CODE>
   * to traverse
   */
  public AlbumListTraversal(ArrayList<Album> alist){
    this(alist, 0);
  }
  
  /**
   * Are there any elements to traverse?
   * The valid range for <CODE>index</CODE> is 
   * <CODE>0 <= index < size</CODE>
   */
  public boolean isEmpty() {
    return this.current == this.alist.size();
  }
  
  /**
   * Produce the first album in this traversal.
   * Throw a runtime exception if there is no data to traverse.
   * @return the first album in this traversal
   */
  public Album getFirst() {
    if (this.isEmpty())
      throw new IllegalUseOfTraversalException
      ("can't retrieve current from an empty list");
    else
      return this.alist.get(current);
  }
  
  /**
   * Produce the the traversal for the rest of the data.
   * Throw a runtime exception if there is no data to traverse.
   * @return the traversal for the rest of this data
   */
   public Traversal<Album> getRest() {
    if (this.isEmpty())
      throw new IllegalUseOfTraversalException
      ("can't advance further in an empty list");
    else
      return new AlbumListTraversal(this.alist, this.current + 1);
  }
}
