/**
 * @(#)Examples.java  7 November 2007
 * Lab 9
 * @author Viera K. Proulx
 */
import java.util.*;
import tester.*;
import interfaces.*;
/**
 * Class to represent sample data and test suite for this lab.
 */
public class Examples implements Testable{

  /** sample <CODE>{@link Album Album}</CODE>. */
  Album soul1 = new Album("Let's Get It On", "Marvin Gaye", 1973);
  /** sample <CODE>{@link Album Album}</CODE>. */
  Album pop1 = new Album("Purple Rain", "Prince", 1984);
  /** sample <CODE>{@link Album Album}</CODE>. */
  Album rock1 = new Album("Are You Experienced", "Jimi Hendrix", 1966);
  /** sample <CODE>{@link Album Album}</CODE>. */
  Album jazz1 = new Album("Miles Smiles", "Miles Davis", 1966);

  /** sample <CODE{@link ArrayList ArrayList}</CODE> of 
   * <CODE>{@link Album Album}</CODE>. */
  ArrayList<Album> albums = new ArrayList<Album>();

  /** initialize the CODE{@link ArrayList ArrayList}</CODE> of 
   * <CODE>{@link Album Album}</CODE>. */
  private void makeList(){
    // remove all elements from the list
    albums.clear();
    
    // now add the ones you wish to include
    albums.add(soul1);
    albums.add(pop1);
    albums.add(rock1);
    albums.add(jazz1);
  }
 
  /** 
   * Tests for the orMap method and its variants
   */
  private void testOrMaps(Tester t){
    makeList();

    // expected true:
    t.test("orMap using ArrayListTraversal",
        true, 
        Algorithms.orMap(new AlbumListTraversal(albums, 0),
            new BeforeYear(1970)));

    t.test("orMap using while loop with Traversal",
        true, 
        Algorithms.orMapWhile(new AlbumListTraversal(albums, 0), 
            new BeforeYear(1970)));

    t.test("orMap using for loop with Traversal",
        true, 
        Algorithms.orMapFor(new AlbumListTraversal(albums, 0), 
            new BeforeYear(1970)));

    t.test("orMap using counted for loop",
        true, 
        Algorithms.orMapForCounted(albums, 
            new BeforeYear(1970)));

    // expected false:
    t.test("orMap using ArrayListTraversal",
        false, 
        Algorithms.orMap(new AlbumListTraversal(albums, 0), 
            new BeforeYear(1950)));

    t.test("orMap using while loop with Traversal",
        false, 
        Algorithms.orMapWhile(new AlbumListTraversal(albums, 0), 
            new BeforeYear(1950)));

    t.test("orMap using for loop with Traversal",
        false, 
        Algorithms.orMapFor(new AlbumListTraversal(albums, 0), 
            new BeforeYear(1950)));

    t.test("orMap using counted for loop",
        false, 
        Algorithms.orMapForCounted(albums, 
            new BeforeYear(1950)));
  }

  /**
   * Tests for the mutating methods
   */ 
  private void testMutation(Tester t){
    ArrayList<Album> albums = new ArrayList<Album>();
    Album unknown;

    //inserts the element at the end of the list
    albums.add(soul1);
    //behaves like cons, inserts the element at the beginning of the list
    albums.add(0, rock1);
    t.test("contains soul1?", true, albums.contains(soul1));
    t.test("is the albums list empty?", false, albums.isEmpty());

    t.test("is rock1 the 1st element?", true, rock1.equals(albums.get(0)));

    //set changes the object in the given index and 
    //returns the object that was there
    unknown = albums.set(1, pop1);
    t.test("is pop1 the 2nd element?", true, pop1.equals(albums.get(1)));
    t.test("before that, was soul1 the 2nd element?", true, soul1.equals(unknown));

    albums.add(jazz1);
    t.test("3 albums in the list?", 3, albums.size());

    albums.remove(jazz1);
    t.test("2 albums in the list?", 2, albums.size());
  }

  /**
   * Run all test cases and report the results.
   * 
   * @param t the <CODE>{@link Tester Tester}</CODE> that performs all tests
   */
  public void tests(Tester t){
    testOrMaps(t);
    testMutation(t);

    t.fullTestReport();
  }

  /**
   * The method that runs the program
   * @param args unused
   */
  public static void main(String[] args) {
    Examples e = new Examples();
    Tester t = new Tester();
    e.tests(t);
    
    System.out.print(t.makeString(e));
  }

}
